// $Id: DBLogin.java,v 1.5 2003/02/17 20:13:23 andy Exp $
package com.tagish.auth;

import java.util.Map;
import java.io.*;
import java.util.*;
import java.sql.*;
import java.math.*;
import java.security.*;
import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.*;
import javax.security.auth.spi.*;

/**
 * Simple database based authentication module.
 *
 * @author Andy Armstrong, <A HREF="mailto:andy@tagish.com">andy@tagish.com</A>
 * @version 1.0.3
 */
public class DBLogin extends SimpleLogin
{
	protected String                dbDriver;
	protected String                dbURL;
	protected String                dbUser;
	protected String                dbPassword;
	protected String                userTable;
	protected String                userField;
        protected String                passwordField;
	protected String                where;

	protected synchronized Vector validateUser(String username, char password[]) throws LoginException
	{
		ResultSet rsu = null, rsr = null;
		Connection con = null;
		PreparedStatement psu = null, psr = null;
		String pass = new String(password);

		try
		{
			Class.forName(dbDriver);
			if (dbUser != null)
			   con = DriverManager.getConnection(dbURL, dbUser, dbPassword);
			else
			   con = DriverManager.getConnection(dbURL);

			psu = con.prepareStatement("SELECT "+userField+","+passwordField+" FROM " + userTable +
									   " WHERE "+userField+"=?" + where);

			psu.setString(1, username);
			rsu = psu.executeQuery();
			if (!rsu.next()) throw new FailedLoginException("Unknown user");
			String user = rsu.getString(1);
			String pwd = rsu.getString(2);
			String md5pass = null;
			try {
				MessageDigest m=MessageDigest.getInstance("MD5");
				m.update(pass.getBytes());
                                byte result[] = m.digest();
				StringBuffer sb = new StringBuffer();
	                        for (int i = 0; i < result.length; i++)
       		                {
                                String ss = Integer.toHexString(result[i]);
                                int length = ss.length();
                                if (length >= 2) {
                                        sb.append(ss.substring(length - 2, length));
                                } else {
                                        sb.append("0");
                                        sb.append(ss);
                                }
                        	}

				md5pass = sb.toString();
			} catch (Exception e) {
				throw new LoginException("Error encoding password (" + e.getMessage() + ")");
			}
			if (!pwd.equals(md5pass)) throw new FailedLoginException("Bad password");
			Vector p = new Vector();
                        p.add(new TypedPrincipal(username, TypedPrincipal.USER));
			p.add(new TypedPrincipal(username, TypedPrincipal.GROUP));
			return p;
		}
		catch (ClassNotFoundException e)
		{
			throw new LoginException("Error reading user database (" + e.getMessage() + ")");
		}
		catch (SQLException e)
		{
			throw new LoginException("Error reading user database (" + e.getMessage() + ")");
		}
		finally
		{
			try {
				if (rsu != null) rsu.close();
				if (psu != null) psu.close();
				if (con != null) con.close();
			} catch (Exception e) { }
		}
	}

	public void initialize(Subject subject, CallbackHandler callbackHandler, Map sharedState, Map options)
	{
		super.initialize(subject, callbackHandler, sharedState, options);

		dbDriver = getOption("dbDriver", null);
		if (dbDriver == null) throw new Error("No database driver named (dbDriver=?)");
		dbURL = getOption("dbURL", null);
		if (dbURL == null) throw new Error("No database URL specified (dbURL=?)");
		dbUser = getOption("dbUser", null);
		dbPassword = getOption("dbPassword", null);
		if ((dbUser == null && dbPassword != null) || (dbUser != null && dbPassword == null))
		   throw new Error("Either provide dbUser and dbPassword or encode both in dbURL");

		userTable    = getOption("userTable",    "User");
		userField = getOption("userField", "uid");
                passwordField = getOption("passwordField",    "password");
		where        = getOption("where",        "");
		if (null != where && where.length() > 0)
			where = " AND " + where;
		else
			where = "";
	}
}
